#-----------------------------------------------------------------------
# File    : pycoco_conda.mak
# Contents: build pycoco dynamic link library (on Windows systems)
#           (for Anaconda distribution)
# Author  : Christian Borgelt
# History : 2012.08.03 file created
#           2013.10.19 module patspec added
#           2013.11.08 properly adapted to Windows/Microsoft Visual C
#           2013.11.14 adapted for Anaconda Python distribution
#           2015.03.04 interrupt signal handling added
#           2015.03.05 preprocessor definitions XXX_ABORT added 
#           2015.08.19 module patred added
#-----------------------------------------------------------------------
!IFNDEF CONDAINC
CONDAINC = "C:\Anaconda\include"
!ENDIF
!IFNDEF CONDALIB
CONDALIB = "C:\Anaconda\libs"
!ENDIF
THISDIR  = ..\..\pycoco\src
UTILDIR  = ..\..\util\src
MATHDIR  = ..\..\math\src
TRACTDIR = ..\..\tract\src
COCODIR  = ..\..\coconad\src

CC       = cl.exe
DEFS     = /D WIN32 /D NDEBUG /D _CONSOLE /D _CRT_SECURE_NO_WARNINGS \
           /D QUIET
CFLAGS   = /nologo /MD /W3 /O2 /GS- $(DEFS) /c
PYINC    = /I $(CONDAINC)
INCS     = /I $(UTILDIR) /I $(MATHDIR) /I $(TRACTDIR) /I $(COCODIR)

LD       = link.exe
LDFLAGS  = /DLL /nologo /incremental:no
LIBS     = /LIBPATH:$(CONDALIB)

HDRS     = $(UTILDIR)\arrays.h   $(UTILDIR)\memsys.h   \
           $(UTILDIR)\symtab.h   $(UTILDIR)\random.h   \
           $(UTILDIR)\sigint.h   $(MATHDIR)\gamma.h    \
           $(TRACTDIR)\tract.h   $(TRACTDIR)\train.h   \
           $(TRACTDIR)\patspec.h $(TRACTDIR)\clomax.h  \
           $(TRACTDIR)\report.h  $(TRACTDIR)\patred.h  \
           $(COCODIR)\cm4seqs.h  $(COCODIR)\ccnpsp.h
OBJS     = arrays.obj memsys.obj idmap.obj random.obj sigint.obj \
           gamma.obj tract.obj train.obj patspec.obj clomax.obj  \
           report.obj predd.obj cm4seqs.obj coconad.obj ccnpsp.obj \
           pycoco.obj $(ADDOBJS)

OVLHDRS  = $(UTILDIR)\arrays.h   $(UTILDIR)\memsys.h   \
           $(UTILDIR)\symtab.h   $(UTILDIR)\random.h   \
           $(UTILDIR)\sigint.h   $(MATHDIR)\gamma.h    \
           $(TRACTDIR)\tract.h   $(TRACTDIR)\train.h   \
           $(TRACTDIR)\patspec.h $(TRACTDIR)\clomax.h  \
           $(TRACTDIR)\report.h  $(TRACTDIR)\patred.h  \
           $(COCODIR)\ccnpsp.h
OVLOBJS  = arrays.obj memsys.obj idmap.obj random.obj sigint.obj \
           gamma.obj tads.obj trnds.obj pspdbl.obj cmdbl.obj     \
           repcmd.obj patred.obj ccn-ovl.obj ccnpsp.obj          \
           pyc-ovl.obj $(ADDOBJS)

PRGS     = coco.pyd ccnovl.pyd

#-----------------------------------------------------------------------
# Build Dynamic Link Library
#-----------------------------------------------------------------------
all:          $(PRGS)

coco.pyd:     $(OBJS) pycoco_conda.mak
	$(LD) $(LDFLAGS) $(LIBS) $(OBJS) /out:$@ /IMPLIB:coco.lib

ccnovl.pyd:   $(OVLOBJS) pycoco_conda.mak
	$(LD) $(LDFLAGS) $(LIBS) $(OBJS) /out:$@ /IMPLIB:ccnovl.lib

#-----------------------------------------------------------------------
# Array Operations
#-----------------------------------------------------------------------
arrays.obj:   $(UTILDIR)\arrays.h $(UTILDIR)\fntypes.h
arrays.obj:   $(UTILDIR)\arrays.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(UTILDIR)\arrays.c /Fo$@

#-----------------------------------------------------------------------
# Memory Management System for Objects of Equal Size
#-----------------------------------------------------------------------
memsys.obj:   $(UTILDIR)\memsys.h
memsys.obj:   $(UTILDIR)\memsys.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(UTILDIR)\memsys.c /Fo$@

#-----------------------------------------------------------------------
# Symbol Table Management
#-----------------------------------------------------------------------
idmap.obj:    $(UTILDIR)\symtab.h $(UTILDIR)\fntypes.h \
              $(UTILDIR)\arrays.h
idmap.obj:    $(UTILDIR)\symtab.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D IDMAPFN $(UTILDIR)\symtab.c /Fo$@

#-----------------------------------------------------------------------
# Random Number Generator Management
#-----------------------------------------------------------------------
random.obj:   $(UTILDIR)\random.h
random.obj:   $(UTILDIR)\random.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(UTILDIR)\random.c /Fo$@

#-----------------------------------------------------------------------
# Interrupt Signal Handling
#-----------------------------------------------------------------------
sigint.obj:   $(UTILDIR)\sigint.h
sigint.obj:   $(UTILDIR)\sigint.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(UTILDIR)\sigint.c /Fo$@

#-----------------------------------------------------------------------
# Gamma Function
#-----------------------------------------------------------------------
gamma.obj:    $(MATHDIR)\gamma.h
gamma.obj:    $(MATHDIR)\gamma.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(MATHDIR)\gamma.c /Fo$@

#-----------------------------------------------------------------------
# Item and Transaction Management
#-----------------------------------------------------------------------
tract.obj:    $(TRACTDIR)\tract.h $(UTILDIR)\arrays.h \
              $(UTILDIR)\symtab.h
tract.obj:    $(TRACTDIR)\tract.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(TRACTDIR)\tract.c /Fo$@

tads.obj:     $(TRACTDIR)\tract.h $(UTILDIR)\arrays.h \
              $(UTILDIR)\symtab.h
tads.obj:     $(TRACTDIR)\tract.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D SUPP=double $(TRACTDIR)\tract.c /Fo$@

#-----------------------------------------------------------------------
# Train/Point Process Management
#-----------------------------------------------------------------------
train.obj:    $(TRACTDIR)\train.h $(TRACTDIR)\tract.h \
              $(UTILDIR)\arrays.h $(UTILDIR)\symtab.h
train.obj:    $(TRACTDIR)\train.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D TRN_SURR $(TRACTDIR)\train.c /Fo$@

trnds.obj:    $(TRACTDIR)\train.h $(TRACTDIR)\tract.h \
              $(UTILDIR)\arrays.h $(UTILDIR)\symtab.h
trnds.obj:    $(TRACTDIR)\train.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D TRN_SURR /D SUPP=double \
              $(TRACTDIR)\train.c /Fo$@

#-----------------------------------------------------------------------
# Item Set Reporter Management
#-----------------------------------------------------------------------
patspec.obj:  $(TRACTDIR)\patspec.h $(TRACTDIR)\tract.h
patspec.obj:  $(TRACTDIR)\patspec.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D PSP_ESTIM /D PSP_TRAIN \
	      $(TRACTDIR)\patspec.c /Fo$@

pspdbl.obj:   $(TRACTDIR)\patspec.h $(UTILDIR)\tabwrite.h
pspdbl.obj:   $(TRACTDIR)\patspec.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D PSP_ESTIM /D PSP_TRAIN \
              /D SUPP=double $(TRACTDIR)\patspec.c /Fo$@

clomax.obj:   $(TRACTDIR)\clomax.h $(TRACTDIR)\tract.h \
              $(UTILDIR)\arrays.h
clomax.obj:   $(TRACTDIR)\clomax.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(TRACTDIR)\clomax.c /Fo$@

cmdbl.obj:    $(TRACTDIR)\clomax.h $(TRACTDIR)\tract.h \
              $(UTILDIR)\arrays.h  $(UTILDIR)\memsys.h
cmdbl.obj:    $(TRACTDIR)\clomax.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D RSUPP=double \
              $(TRACTDIR)\clomax.c /Fo$@

report.obj:   $(TRACTDIR)\report.h $(TRACTDIR)\clomax.h \
              $(TRACTDIR)\tract.h  $(UTILDIR)\arrays.h  \
              $(UTILDIR)\symtab.h
report.obj:   $(TRACTDIR)\report.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D ISR_PATSPEC /D ISR_CLOMAX \
              /D ISR_NONAMES $(TRACTDIR)\report.c /Fo$@

repcmd.obj:   $(TRACTDIR)\report.h $(TRACTDIR)\tract.h \
              $(UTILDIR)\arrays.h  $(UTILDIR)\symtab.h
repcmd.obj:   $(TRACTDIR)\report.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D ISR_PATSPEC /D ISR_CLOMAX \
              /D ISR_NONAMES /D RSUPP=double $(TRACTDIR)\report.c /Fo$@

#-----------------------------------------------------------------------
# Pattern Set Reduction Functions
#-----------------------------------------------------------------------
patred.obj:   $(HDRS) $(TRACTDIR)\patred.h
patred.obj:   $(TRACTDIR)\patred.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(TRACTDIR)\patred.c /Fo$@

predd.obj:    $(HDRS) $(TRACTDIR)\patred.h
predd.obj:    $(TRACTDIR)\patred.c pycoco.mak
	$(CC) $(CFLAGS) $(INCS) /D RSUPP=double \
              $(TRACTDIR)\patred.c /Fo$@

#-----------------------------------------------------------------------
# Prefix Tree Management for Closed and Maximal Sequences
#-----------------------------------------------------------------------
cm4seqs.obj:  $(HDRS)
cm4seqs.obj:  $(COCODIR)\cm4seqs.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(COCODIR)\cm4seqs.c /Fo$@

#-----------------------------------------------------------------------
# CoCoNAD
#-----------------------------------------------------------------------
coconad.obj:  $(HDRS) $(COCODIR)\coconad.h \
              $(UTILDIR)\fntypes.h $(UTILDIR)\error.h
coconad.obj:  $(COCODIR)\coconad.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_ABORT $(COCODIR)\coconad.c /Fo$@

ccn-ovl.obj:  $(HDRS) $(COCODIR)\coconad.h $(UTILDIR)\fntypes.h
ccn-ovl.obj:  $(COCODIR)\coconad.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_ABORT /D CCN_OVERLAP \
              /D SUPP=double $(COCODIR)\coconad.c /Fo$@

#-----------------------------------------------------------------------
# Pattern Spectrum Functions
#-----------------------------------------------------------------------
ccnpsp.obj:   $(HDRS) $(UTILDIR)\fntypes.h $(COCODIR)\coconad.h \
              $(COCODIR)\ccnpsp.h
ccnpsp.obj:   $(COCODIR)\coconad.c $(COCODIR)\ccnpsp.c pycoco.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_ABORT $(COCODIR)\ccnpsp.c /Fo$@

#-----------------------------------------------------------------------
# Python Stuff
#-----------------------------------------------------------------------
pycoco.obj:   $(HDRS) $(COCODIR)\coconad.h
pycoco.obj:   pycoco.c pycoco_conda.mak
	$(CC) $(CFLAGS) $(INCS) $(PYINC) pycoco.c /Fo$@

pyc-ovl.obj:  $(HDRS) $(COCODIR)\coconad.h
pyc-ovl.obj:  pycoco.c pycoco.mak
	$(CC) $(CFLAGS) $(INCS) $(PYINC) \
              /D CCN_OVERLAP /D SUPP=double pycoco.c /Fo$@

#-----------------------------------------------------------------------
# Install
#-----------------------------------------------------------------------
install:
	-@copy *.pyd ..\..\..\bin

#-----------------------------------------------------------------------
# Clean up
#-----------------------------------------------------------------------
clean:
	-@erase /Q *~ *.obj *.idb *.pch *.exp *.lib $(PRGS)
